
// WIRE.CPP
//   A set of functions to create translation matrices for
// various wireframe transformation functions, to trans-
// form a wireframe object using a matrix concatenated
// from those matrices, and to draw a wireframe shape on
// the mode 13h display
//
// Copyright 1993 by Christopher Lampton and
// The Waite Group Press

#include  <math.h>
#include "cwire.h"

double matrix[4][4];         // Master transformation matrix
double smat[4][4];				    // Scaling matrix
double zmat[4][4];				    // Z rotation matrix
double xmat[4][4];				    // X rotation matrix
double ymat[4][4];				    // Y rotation matrix
double tmat[4][4];           // Translation matrix


void matmult(double result[4][4],double mat1[4][4],
              double mat2[4][4]);
void matcopy(double dest[4][4],double source[4][4]);



// Transformation functions:

void inittrans()
{

// Initialize master transformation matrix to the
//  identity matrix

  matrix[0][0]=1; matrix[0][1]=0; matrix[0][2]=0;
    matrix[0][3]=0;
  matrix[1][0]=0; matrix[1][1]=1; matrix[1][2]=0;
    matrix[1][3]=0;
  matrix[2][0]=0; matrix[2][1]=0; matrix[2][2]=1;
    matrix[2][3]=0;
  matrix[3][0]=0; matrix[3][1]=0; matrix[3][2]=0;
    matrix[3][3]=1;
}


void rotate(double ax,double ay,double az)
{

  // Create three rotation matrices that will rotate an
  // object AX radians on the X axis, AY radians on the
  // Y axis and AZ radians on the Z axis

  double mat1[4][4];
  double mat2[4][4];

	// Initialize X rotation matrix:

	xmat[0][0]=1; xmat[0][1]=0; xmat[0][2]=0; xmat[0][3]=0;
  xmat[1][0]=0; xmat[1][1]=cosl(ax); xmat[1][2]=sinl(ax);
    xmat[1][3]=0;
	xmat[2][0]=0; xmat[2][1]=-sinl(ax); xmat[2][2]=cosl(ax);
    xmat[2][3]=0;
	xmat[3][0]=0; xmat[3][1]=0; xmat[3][2]=0; xmat[3][3]=1;

  // Concatenate this matrix with master matrix:

  matmult(mat1,xmat,matrix);

  // Initialize Y rotation matrix:

	ymat[0][0]=cosl(ay); ymat[0][1]=0; ymat[0][2]=-sinl(ay);
    ymat[0][3]=0;
  ymat[1][0]=0; ymat[1][1]=1; ymat[1][2]=0; ymat[1][3]=0;
	ymat[2][0]=sinl(ay); ymat[2][1]=0; ymat[2][2]=cosl(ay);
   ymat[2][3]=0;
	ymat[3][0]=0; ymat[3][1]=0; ymat[3][2]=0; ymat[3][3]=1;

  // Concatenate this matrix with master matrix:

  matmult(mat2,ymat,mat1);

	// Initialize Z rotation matrix:

	zmat[0][0]=cosl(az); zmat[0][1]=sinl(az); zmat[0][2]=0;
    zmat[0][3]=0;
	zmat[1][0]=-sinl(az); zmat[1][1]=cosl(az); zmat[1][2]=0;
    zmat[1][3]=0;
	zmat[2][0]=0; zmat[2][1]=0; zmat[2][2]=1; zmat[2][3]=0;
	zmat[3][0]=0; zmat[3][1]=0; zmat[3][2]=0; zmat[3][3]=1;

  // Concatenate this matrix with master matrix:

  matmult(matrix,zmat,mat2);
}

void translate(double xt,double yt,double zt)
{

// Create a translation matrix that will translate an
// object an X distance of XT, a Y distance of YT, and a
// Z distance of ZT from the screen origin

  double mat[4][4];

  tmat[0][0]=1; tmat[0][1]=0; tmat[0][2]=0; tmat[0][3]=0;
  tmat[1][0]=0; tmat[1][1]=1; tmat[1][2]=0; tmat[1][3]=0;
  tmat[2][0]=0; tmat[2][1]=0; tmat[2][2]=1; tmat[2][3]=0;
  tmat[3][0]=xt; tmat[3][1]=yt; tmat[3][2]=zt; tmat[3][3]=1;

  // Concatenate with master matrix:

  matmult(mat,matrix,tmat);
  matcopy(matrix,mat);
}

void matmult(double result[4][4],double mat1[4][4],
              double mat2[4][4])
{
 int i,j,k;
// Multiply matrix MAT1 by matrix MAT2,
//  returning the result in RESULT

  for ( i=0; i<4; i++)
    for ( j=0; j<4; j++) {
      result[i][j]=0;
      for ( k=0; k<4; k++)
        result[i][j]+=mat1[i][k] * mat2[k][j];
    }
}

void matcopy(double dest[4][4],double source[4][4])
{
  int i,j;
// Copy matrix SOURCE to matrix DEST

  for ( i=0; i<4; i++)
    for ( j=0; j<4; j++)
      dest[i][j]=source[i][j];
}
